# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class SPSSCrossTab(Template):
    'Create a cross-tabulation in SPSS (equivalent to Analyse -> Descriptive Statistics -> Crosstabs... in SPSS 23)'

    __version__ = '1.0.0'

    tags = ['SPSS', 'Crosstab']
    engines = ['SPSS_script']

    inputs = '''
rowvar = DataVector('Row variable: ')
colvar = DataVector('Column variable: ')
exactopts = Boolean('Use Exact? ', default = False)
cellopts = Boolean('Specify Cell options? ', default = False)
if cellopts:
    countopts = Boolean('Count options? ', default = False)
    if countopts:
        count = Boolean('Observed cell counts? ', default = True)
        expected = Boolean('Expected counts? ', default = False)
    percentopts = Boolean('Percentage options? ', default = False)
    if percentopts:
        row = Boolean('Row percentages? ', default = False)
        col = Boolean('Column percentages? ', default = False)
        total = Boolean('Two-way table total percentages? ', default = False)
mh = Boolean('Describe Mantel-Haenszel test? ', default = False)
'''

    spssscript = '''
CROSSTABS 
  /TABLES=${rowvar} BY ${colvar}
  /STATISTIC= CHISQ 
% if cellopts:
  /CELLS=\\
% if countopts:
% if count:
 COUNT\\
% endif
% if expected:
 EXPECTED\\
% endif
% else:
 COUNT\\
% endif
% if percentopts:
% if row:
 ROW\\
% endif
% if col:
 COLUMN\\
% endif
% if total:
 TOTAL\\
% endif
% endif
% endif

% if exactopts:
  /METHOD= EXACT
% endif
.
'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import numpy
import random
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

caseout =  etree.parse(BytesIO(outputs['Table_Case Processing Summary.xml'].content))
print(etree.tostring(caseout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/@text)"
casetext = caseout.xpath(path, namespaces=ns)
print(casetext)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Valid']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nvalid = int(caseout.xpath(path, namespaces=ns))
print(Nvalid)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Missing']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nmissing = int(caseout.xpath(path, namespaces=ns))
print(Nmissing)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Ntotal = int(caseout.xpath(path, namespaces=ns))
print(Ntotal)

chiout = etree.parse(BytesIO(outputs['Table_Chi Square Tests.xml'].content))
print(etree.tostring(chiout, pretty_print=True))

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@number)"
pearval_num = str2float(chiout.xpath(path, namespaces=ns))
print(pearval_num)

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
pearval_str = chiout.xpath(path, namespaces=ns)
print(pearval_str)

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/x:footnote/x:note/@text)"
pearfoot = chiout.xpath(path, namespaces=ns)
print(pearfoot)

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='df']/x:cell/@number)"
peardf = int(chiout.xpath(path, namespaces=ns))
print(peardf)

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@number)"
pearasig2_num = str2float(chiout.xpath(path, namespaces=ns))
print(pearasig2_num)

path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@text)"
pearasig2_str = chiout.xpath(path, namespaces=ns)
print(pearasig2_str)

if exactopts:
    path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@number)"
    pearesig2_num = str2float(chiout.xpath(path, namespaces=ns))
    print(pearesig2_num)

    path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@text)"
    pearesig2_str = chiout.xpath(path, namespaces=ns)
    print(pearesig2_str)

    #path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@number)"
    #pearesig1_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(pearesig1_num)

    #path = "string(//x:group[@text='A']/x:category[@text='Pearson Chi-Square']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@text)"
    #pearesig1_str = chiout.xpath(path, namespaces=ns)
    #print(pearesig1_str)

    #path = "string(//x:group[@text='A']/x:category[@text='Continuity Correction']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@number)"
    #ccval_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(ccval_num)

    #path = "string(//x:group[@text='A']/x:category[@text='Continuity Correction']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
    #ccval_str = chiout.xpath(path, namespaces=ns)
    #print(ccval_str)

    #path = "string(//x:group[@text='A']/x:category[@text='Continuity Correction']/x:dimension[@text='Values']/x:category[@text='df']/x:cell/@number)"
    #ccdf = int(chiout.xpath(path, namespaces=ns))
    #print(ccdf)

    #path = "string(//x:group[@text='A']/x:category[@text='Continuity Correction']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@number)"
    #ccasig2_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(ccasig2_num)

    #path = "string(//x:group[@text='A']/x:category[@text='Continuity Correction']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@text)"
    #ccasig2_str = chiout.xpath(path, namespaces=ns)
    #print(ccasig2_Str)

path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@number)"
llval_num = str2float(chiout.xpath(path, namespaces=ns))
print(llval_num)

path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
llval_str = chiout.xpath(path, namespaces=ns)
print(llval_str)

path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:category[@text='df']/x:cell/@number)"
lldf = int(chiout.xpath(path, namespaces=ns))
print(lldf)

path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@number)"
llasig2_num = str2float(chiout.xpath(path, namespaces=ns))
print(llasig2_num)

path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@text)"
llasig2_str = chiout.xpath(path, namespaces=ns)
print(llasig2_str)

if exactopts:
    path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@number)"
    llesig2_num = str2float(chiout.xpath(path, namespaces=ns))
    print(llesig2_num)

    path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@text)"
    llesig2_str = chiout.xpath(path, namespaces=ns)
    print(llesig2_str)

    #path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@number)"
    #llesig1_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(llesig1_num)

    #path = "string(//x:group[@text='B']/x:category[@text='Likelihood Ratio']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@text)"
    #llesig1_str = chiout.xpath(path, namespaces=ns)
    #print(llesig1_str)
 
    #path = "string(//x:group[@text='B']/x:category[contains(@text, 'Fisher')]/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@number)"
    #fishval_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(fishval_num)

    #path = "string(//x:group[@text='B']/x:category[contains(@text, 'Fisher')]/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
    #fishval_str = chiout.xpath(path, namespaces=ns)
    #print(fishval_str)

    #path = "string(//x:group[@text='B']/x:category[contains(@text, 'Fisher')]/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@number)"
    #fishesig2_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(fishesig2_num)

    #path = "string(//x:group[@text='B']/x:category[contains(@text, 'Fisher')]/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@text)"
    #fishesig2_str = chiout.xpath(path, namespaces=ns)
    #print(fishesig2_str)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@number)"
linassocval_num = str2float(chiout.xpath(path, namespaces=ns))
print(linassocval_num)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
linassocval_str = chiout.xpath(path, namespaces=ns)
print(linassocval_str)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/x:footnote/x:note/@text)"
linassocfoot = chiout.xpath(path, namespaces=ns)
print(linassocfoot)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='df']/x:cell/@number)"
linassocdf = int(chiout.xpath(path, namespaces=ns))
print(linassocdf)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@number)"
linassocasig2_num = str2float(chiout.xpath(path, namespaces=ns))
print(linassocasig2_num)

path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:category[@text='Asymptotic Significance (2-sided)']/x:cell/@text)"
linassocasig2_str = chiout.xpath(path, namespaces=ns)
print(linassocasig2_str)

if exactopts:
    path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@number)"
    linassocesig2_num = str2float(chiout.xpath(path, namespaces=ns))
    print(linassocesig2_num)

    path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (2-sided)']/x:cell/@text)"
    linassocesig2_str = chiout.xpath(path, namespaces=ns)
    print(linassocesig2_str)

    #path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@number)"
    #linassocesig1_num = str2float(chiout.xpath(path, namespaces=ns))
    #print(linassocesig1_num)

    #path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Exact Sig. (1-sided)']/x:cell/@text)"
    #linassocesig1_str = chiout.xpath(path, namespaces=ns)
    #print(linassocesig1_str)

    path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Point Probability']/x:cell/@number)"
    linassocpp_num = str2float(chiout.xpath(path, namespaces=ns))
    print(linassocpp_num)

    path = "string(//x:category[@text='Linear-by-Linear Association']/x:dimension[@text='Values']/x:group[@text='C']/x:category[@text='Point Probability']/x:cell/@text)"
    linassocpp_str = chiout.xpath(path, namespaces=ns)
    print(linassocpp_str)

path = "string(//x:category[@text='N of Valid Cases']/x:dimension[@text='Values']/x:category[@text='Value']/x:cell/@text)"
Nvalidcase = int(chiout.xpath(path, namespaces=ns))
print(Nvalidcase)

crosstabout = etree.parse(BytesIO(outputs['Table_Crosstabulation.xml'].content))
print(etree.tostring(crosstabout, pretty_print=True))

path = "string(//x:dimension[@axis='layer']/x:category[@text='Count']/x:dimension[@axis='row']/@label)"
rowlabel = crosstabout.xpath(path, namespaces=ns)
print(rowlabel)

path = "string(//x:dimension[@axis='layer']/x:category[@text='Count']/x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/@label)"
collabel = crosstabout.xpath(path, namespaces=ns)
print(collabel)

path = "//x:dimension[@axis='row']/x:group/x:category/@label"
rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
if rowcats == []:
     path = "//x:dimension[@axis='row']/x:group/x:category/@text"
     rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
 
print(rowcats)

if cellopts:
    # FIXME: The following don't match if only observed count is selected, as SPSS doesn't create the 'Count' category (this should probably use the "else" section)
    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/@label"
    colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
    if colcats == []:
        path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/@text"
        colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
    print(colcats)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    counts = numpy.array([int(x) for x in crosstabout.xpath(path, namespaces=ns)])
    counts = counts.reshape([len(rowcats), len(colcats)])
    print(counts)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@number"
    countcoltots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(countcoltots)

    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    countrowtots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(countrowtots)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    ecounts_num = numpy.array([str2float(x) for x in crosstabout.xpath(path, namespaces=ns)])
    ecounts_num = ecounts_num.reshape([len(rowcats), len(colcats)])
    print(ecounts_num)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
    ecounts_str = numpy.array(crosstabout.xpath(path, namespaces=ns))
    ecounts_str = ecounts_str.reshape([len(rowcats), len(colcats)])
    print(ecounts_str)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@number"
    ecountcoltots_num = [str2float(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(ecountcoltots_num)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@text"
    ecountcoltots_str = crosstabout.xpath(path, namespaces=ns)
    print(ecountcoltots_str)

    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    ecountrowtots_num = [str2float(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(ecountrowtots_num)

    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Expected Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
    ecountrowtots_str = crosstabout.xpath(path, namespaces=ns)
    print(ecountrowtots_str)

    path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
    percents = [str(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(percents)

    path = "//x:category[@text='% of Total']/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
    percentrowtots = [str(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(percentrowtots)

else:
    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='column']/x:group/x:category/@label"
    colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
    print(colcats)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    counts = numpy.array([int(x) for x in crosstabout.xpath(path, namespaces=ns)])
    counts = counts.reshape([len(rowcats), len(colcats)])
    print(counts)

    path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@number"
    countcoltots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(countcoltots)

    path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
    countrowtots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
    print(countrowtots)

varname1short = str(rowvar)
varname1long = str(rowvar)
if rowlabel != '':
    varname1long = rowlabel + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(colvar)
varname2long = str(colvar)
if collabel != '':
    varname2long = collabel + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

rowfortext = random.randint(0, len(rowcats) - 1)
colfortext = random.randint(0, len(colcats) - 1)
rowqfortext = random.randint(0, len(rowcats) - 1)
colqfortext = random.randint(0, len(colcats) - 1)

introhtml = '<h2>The Chi-squared test</h2>'
introhtml += '<p>'
introhtml += 'In this practical we are going to investigate how to perform a chi-squared test using SPSS. '
introhtml += 'A chi-squared test is used when we have two categorical variables measured for all observations in a dataset and we want to test if the variables are related or independent. '
introhtml += 'Independent here means that the category observed for one variable does not depend on the category observed for the other variable. '
introhtml += 'To run a single test in SPSS requires that your dataset has two separate columns containing the two (categorical) variables to be tested. '
introhtml += '</p>' 

outputs['introduction.html'] = HTMLOutput(introhtml, "Intoduction text")

summaryprachtml = '<p>'
summaryprachtml += 'Below you will see instructions to perform the chisquared test in SPSS. '
summaryprachtml += 'For the chi-squared test we will use SPSS crosstabs options that we have also looked at in our tabulation practical. '
summaryprachtml += 'If you follow the instructions you will see the three tabular outputs that are embedded in the explanations below. '
summaryprachtml += '</p>'

summaryinstrhtml = '<ul class = "instructions">'
summaryinstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
summaryinstrhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
summaryinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
summaryinstrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Row(s):</strong> box.</li>'
summaryinstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Column(s):</strong> box.</li>'
summaryinstrhtml += '<li>Click on the <strong>Statistics...</strong> button.</li>'
summaryinstrhtml += '<li>In the window that appears click on the <strong>Chi-square</strong> tickbox.</li>'
summaryinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
if exactopts:
    summaryinstrhtml += '<li>Click on the <strong>Exact...</strong> button.</li>'
    summaryinstrhtml += '<li>In the window that appears click on the <strong>Exact</strong> button.</li>'
    summaryinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
if cellopts:
    done = False
    if countopts:
        if not count:
            summaryinstrhtml += '<li>Click on the <strong>Cells...</strong>} button.</li>'
            summaryinstrhtml += '<li>In the window that appears:</li>'
            done = True
            summaryinstrhtml += '<li>Under counts click on the <strong>Observed</strong> tick box to remove it.</li>'
        if expected:
            if not done:
                summaryinstrhtml += '<li>Click on the <strong>Cells...</strong> button.</li>'
                summaryinstrhtml += '<li>In the window that appears:</li>'
                done = True
            summaryinstrhtml += '<li>Under counts click on the <strong>Expected</strong> tick box to include it.</li>'
    if percentopts:
        if row:
            if not done:
                summaryinstrhtml += '<li>Click on the <strong>Cells...</strong> button.</li>'
                summaryinstrhtml += '<li>In the window that appears:</li>'
                done = True
            summaryinstrhtml += '<li>Under percentages click on the <strong>Row</strong> tick box to include it.</li>'
        if col:
            if not done:
                summaryinstrhtml += '<li>Click on the <strong>Cells...</strong> button.</li>'
                summaryinstrhtml += '<li>In the window that appears:</li>'
                done = True
            summaryinstrhtml += '<li>Under percentages click on the <strong>Column</strong> tick box to include it.</li>'
        if total:
            if not done:
                summaryinstrhtml += '<li>Click on the <strong>Cells...</strong> button.</li>'
                summaryinstrhtml += '<li>In the window that appears:</li>'
                done = True
            summaryinstrhtml += '<li>Under percentages click on the <strong>Total</strong> tick box to include it.</li>'
    if done: 
        summaryinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
summaryinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
summaryinstrhtml += '</ul>'

summaryprachtml += summaryinstrhtml

summaryprachtml += '<p>'
summaryprachtml += 'The first table looks at which of the observations have non-missing values for both the two variables to be considered. '
summaryprachtml += '</p>'

summaryprachtml += outputs['Table_Case Processing Summary.htm'].content

summaryprachtml += '<p>'
if Nmissing == 0:
    summaryprachtml += 'Here we see that there are ' + str(Ntotal) + ' observations and none of these observations has missing data and so all can be used in the test. '
else:
    summaryprachtml += 'Here we see that there are ' + str(Ntotal) + ' observations of which ' + str(Nmissing) + ' are missing resulting in ' + str(Nvalid) + ' that can be used in the test. '
summaryprachtml += '</p>'
outputs['casesummary.html'] = HTMLOutput(summaryprachtml, "Case processing text")

summaryquesthtml = '<p>'
summaryquesthtml += 'Use the Crosstabs window to perform a chi-squared test on the variable ' + varname1short + ' and ' + varname2short + ' and answer the following questions: '
summaryquesthtml += '</p>'
summaryquesthtml += '<ul>'
summaryquesthtml += '<li>Question: In the first output we see the numbers of observations and missing data. How many data points are used in the test?</li>'
summaryquesthtml += '</ul>'

outputs['casesummaryquestions.html'] = HTMLOutput(summaryquesthtml, "Case processing questions text")

summarysolhtml = 'The SPSS instructions are as follows: '
summarysolhtml += summaryinstrhtml

summarysolhtml += '<ul>'
summarysolhtml += '<li>Question: In the first output we see the numbers of observations and missing data. How many data points are used in the test?</li>'
summarysolhtml += '</ul>'
summarysolhtml += '<p>'
summarysolhtml += 'Solution: The output from SPSS is as follows:'
summarysolhtml += '</p>'
summarysolhtml += outputs['Table_Case Processing Summary.htm'].content
summarysolhtml += '<p>'
if Nmissing == 0:
    summarysolhtml += 'Here we see that there are ' + str(Ntotal) + ' observations and none of these observations has missing data and so all can be used in the test. '
else:
    summarysolhtml += 'Here we see that there are ' + str(Ntotal) + ' observations of which ' + str(Nmissing) + ' are missing resulting in ' + str(Nvalid) + ' that can be used in the test. '
summarysolhtml += '</p>'
outputs['casesummaryanswers.html'] = HTMLOutput(summarysolhtml, "Case processing answers text")

crosstabprachtml = '<p>'
crosstabprachtml += 'The second tabular output contains the cross-tabulation of the two variables. '
crosstabprachtml += 'Here different levels of ' + varname1short + ' are allocated to different rows whilst different values of ' + varname2short + ' are allocated to different columns and each observation adds to the count of a particular cell in the table. '
crosstabprachtml += '</p>'

crosstabprachtml += outputs['Table_Crosstabulation.htm'].content

crosstabprachtml += '<p>'
crosstabprachtml += 'Here we see the observed counts for the different combinations of ' + varname1short + ' and ' + varname2short + '. '
crosstabprachtml += 'So for example there are ' + str(counts[rowfortext, colfortext]) + ' observations where ' + varname1short + ' response is ' + rowcats[rowfortext] + ' and the ' + varname2short + ' response is ' + colcats[colfortext] + '. '
crosstabprachtml += 'This is out of a total of ' + str(countcoltots[rowfortext]) + ' observations where ' + varname1short + ' is ' + rowcats[rowfortext] + ' and ' + str(countrowtots[colfortext]) + ' observations where ' + varname2short + ' is ' + colcats[colfortext] + '. '

if cellopts:
    if countopts:
        if expected:
            crosstabprachtml += 'Under the model of independence we expect to see ' + ecounts_str[rowfortext, colfortext] + ' observations where ' + varname1short + ' is ' + rowcats[rowfortext] + ' and ' + varname2short + ' is ' + colcats[colfortext] + '. '
            if ecounts_num[rowfortext, colfortext] > counts[rowfortext, colfortext]:
                crosstabprachtml += 'This means there are ' + str(round(ecounts_num[rowfortext, colfortext] - counts[rowfortext, colfortext], 1)) + ' fewer observations where ' + varname1short + ' is ' + rowcats[rowfortext] + ' and ' + varname2short + ' is ' + colcats[colfortext] + ' than expected. '
            else:
                crosstabprachtml += 'This means there are ' + str(round(counts[rowfortext, colfortext] - ecounts_num[rowfortext, colfortext], 1)) + ' more observations where ' + varname1short + ' is ' + rowcats[rowfortext] + ' and ' + varname2short + ' is ' + colcats[colfortext] + ' than expected. '
            crosstabprachtml += 'We can look at similar differences between the other observed and expected counts and the chi-squared test looks at whether these differences are simply chance occurrences or statistically significant. '

crosstabprachtml += '</p>'

outputs['crosstabulation.html'] = HTMLOutput(crosstabprachtml, "Crosstab text")

crosstabquesthtml = '<ul>'
crosstabquesthtml += '<li>Question: Looking at the second output how many observations are there where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '?</li>'
if cellopts:
    if countopts:
        if expected:
            crosstabquesthtml += '<li>Question: How many observations are expected under the null hypothesis of independence where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '?</li>'
crosstabquesthtml += '</ul>'

outputs['crosstabulationquestions.html'] = HTMLOutput(crosstabquesthtml, "Crosstab questions text")

crosstabsolhtml = '<ul>'
crosstabsolhtml += '<li>Question: Looking at the second output how many observations are there where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '?</li>'
crosstabsolhtml += '</ul>'
crosstabsolhtml += '<p>'
crosstabsolhtml += 'Solution: The output from SPSS is as follows:'
crosstabsolhtml += '</p>'
crosstabsolhtml += outputs['Table_Crosstabulation.htm'].content
crosstabsolhtml += '<p>'
crosstabsolhtml += 'There are ' + str(counts[rowqfortext, colqfortext]) + ' observations where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '. '
crosstabsolhtml += 'This is out of a total of ' + str(countcoltots[rowqfortext]) + ' observations where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + str(countrowtots[colqfortext]) + ' observations where ' + varname2short + ' is ' + colcats[colqfortext] + '. '
crosstabsolhtml += '</p>'
if cellopts:
    if countopts:
        if expected:
            crosstabsolhtml += '<ul>'
            crosstabsolhtml += '<li>Question: How many observations are expected under the null hypothesis of independence where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '?</li>'
            crosstabsolhtml += '</ul>'
            crosstabsolhtml += '<p>'
            crosstabsolhtml += 'Solution: The output from SPSS is as follows:'
            crosstabsolhtml += '</p>'
            crosstabsolhtml += outputs['Table_Crosstabulation.htm'].content
            crosstabsolhtml += '<p>'
            crosstabsolhtml += 'Under the model of independence we expect to see ' + ecounts_str[rowqfortext, colqfortext] + ' observations where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + '. '
            if ecounts_num[rowqfortext, colqfortext] > counts[rowqfortext, colqfortext]:
                crosstabsolhtml += 'This means there are ' + str(round(ecounts_num[rowqfortext, colqfortext] - counts[rowqfortext, colqfortext], 1)) + ' fewer observations where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + ' than expected. '
            else:
                crosstabsolhtml += 'This means there are ' + str(round(counts[rowqfortext, colqfortext] - ecounts_num[rowqfortext, colqfortext], 1)) + ' more observations where ' + varname1short + ' is ' + rowcats[rowqfortext] + ' and ' + varname2short + ' is ' + colcats[colqfortext] + ' than expected. '
            crosstabsolhtml += '</p>' 

outputs['crosstabulationanswers.html'] = HTMLOutput(crosstabsolhtml, "Crosstab answers text")

chisqprachtml = '<p>'
chisqprachtml += 'The third tabular output contains the information on test statistics for testing the hypothesis of no association / relationship between the two variables. '
chisqprachtml += '</p>'

chisqprachtml += outputs['Table_Chi Square Tests.htm'].content

chisqprachtml += '<p>'
chisqprachtml += 'SPSS gives several tests for significance and we will first focus on the Pearson Chi-Square test. '
chisqprachtml += 'This test begins by forming the Pearson test statistic which asymptotically is formed from the observed and expected cell counts. '
chisqprachtml += 'For each cell the difference between the observed and expected counts is found and squared. This positive number is then divided by the expected count to account for different sizes of cells. '
chisqprachtml += 'Having constructed this value for each cell these are summed across all cells to give our test statistic which here is ' + pearval_str + '. '
chisqprachtml += 'This statistic follows a chi-squared distribution under the null hypothesis with degrees of freedom equal to (rows-1)x(columns-1) which in this case equals ' + str(peardf) + '. '
chisqprachtml += 'The statistic is then compared with the appropriate Chi-Squared distribution and this results in an asymptotic (2-sided) p value which has value ' 
if pearasig2_num < 0.001:
    chisqprachtml += pearasig2_str + '(reported as p &lt; .001). '
else:
    chisqprachtml += pearasig2_str + '. '
if pearasig2_num > 0.05:
    chisqprachtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two variables are independent. '
else:
    chisqprachtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two variables are independent and there is therefore some relationship between the variables. '
if exactopts:
    chisqprachtml += 'The chi-squared test is an approximate test but it is also possible to fit an exact test which involves looking at all possible tables with the same row and column totals and establishing how likely the observed table is. '
    chisqprachtml += 'This is quoted as a p value and here the value is ' + pearesig2_str + '. '
    if pearasig2_num > 0.05 and pearesig2_num > 0.05:
        chisqprachtml += 'Here the exact test agrees with the asymptotic result that we cannot reject the null hypothesis. '
    if pearasig2_num <= 0.05 and pearesig2_num <= 0.05:
        chisqprachtml += 'Here the exact test agrees with the asymptotic result that we can reject the null hypothesis. '
    if pearasig2_num <= 0.05 and pearesig2_num > 0.05:
        chisqprachtml += 'Here the exact test disagrees with the asymptotic result and states that we cannot in fact reject the null hypothesis. '
    if pearasig2_num > 0.05 and pearesig2_num <= 0.05:
        chisqprachtml += 'Here the exact test disagrees with the asymptotic result and states that we can in fact reject the null hypothesis. '
chisqprachtml += '</p>'

chisqprachtml += '<p>'
if mh:
    chisqprachtml += 'There is a second test called the Mantel-Haenszel test which considers the two variables to be ordinal (ordered category) rather than nominal variables and again tests for a relationship with the null hypothesis being no relationship between the variables. '
    chisqprachtml += 'SPSS gives the output for this test in the row marked Linear-by-Linear Association. As with the chi-squared test this test has a test statistic which here is ' + linassocval_str + ' and degrees of freedom ' + str(linassocdf) + '. '
    chisqprachtml += 'This results in a p value given in SPSS of ' + linassocasig2_str + '. '
    if linassocasig2_num > 0.05:
        chisqprachtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two variables are independent. '
    else:
        chisqprachtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two variables are independent and there is therefore a linear relationship between the variables. '
    if linassocasig2_num < 0.001:
        chisqprachtml += 'Although SPSS quotes the p value as ' + linassocval_str + ' it is not exactly zero and is in fact simply smaller than 0.001 as SPSS is quoting the first 3 decimal places. '
    if exactopts:
        chisqprachtml += 'The Mantel-Haenszel test is an approximate test but it is also possible to fit an exact test which involves looking at all possible tables with the same row and column totals and establishing how likely the observed table is. '
        chisqprachtml += 'This is quoted as a p value and here the value is ' + linassocesig2_str + '. '
        if linassocasig2_num > 0.05 and linassocesig2_num > 0.05:
            chisqprachtml += 'Here the exact test agrees with the asymptotic result that we cannot reject the null hypothesis. '
        if linassocasig2_num <= 0.05 and linassocesig2_num <= 0.05:
            chisqprachtml += 'Here the exact test agrees with the asymptotic result that we can reject the null hypothesis. '
        if linassocasig2_num <= 0.05 and linassocesig2_num > 0.05:
            chisqprachtml += 'Here the exact test disagrees with the asymptotic result and states that we cannot in fact reject the null hypothesis. '
        if linassocasig2_num > 0.05 and linassocesig2_num <= 0.05:
            chisqprachtml += 'Here the exact test disagrees with the asymptotic result and states that we can in fact reject the null hypothesis. '

else:
    chisqprachtml += 'The lines labelled Likelihood Ratio and Linear-by-Linear Association are alternative tests of independence that are less commonly used than the Chi-square test and we will not cover them here. '
chisqprachtml += '</p> '
outputs['chisquared.html'] = HTMLOutput(chisqprachtml, "Chi-squared text")

chisqquesthtml = '<ul>'
chisqquesthtml += '<li>Question: Looking at the third output does the asymptotic Chi-squared test suggest a significant relationship between the variables?</li>'
if exactopts:
    chisqquesthtml += '<li>Question: Does the exact Chi-squared test agree with the asymptotic result?</li>'
if mh:
    chisqquesthtml += '<li>Question: Does the Mantel-Haenszel test suggest a significant relationship between the (ordinal) variables?</li>'
chisqquesthtml += '</ul>'

outputs['chisquaredquestions.html'] = HTMLOutput(chisqquesthtml, "Chi-squared questions text")

chisqsolhtml = '<ul>'
chisqsolhtml += '<li>Question: Looking at the third output does the asymptotic Chi-squared test suggest a significant relationship between the variables?</li>'
chisqsolhtml += '</ul>'
chisqsolhtml += '<p>'
chisqsolhtml += 'Solution: The output from SPSS is as follows:'
chisqsolhtml += '</p>'
chisqsolhtml += outputs['Table_Chi Square Tests.htm'].content
chisqsolhtml += '<p>'
chisqsolhtml += 'The relationship between the categorical variables, ' + varname1short + ' and ' + varname2short + ' was examined to look for associations. '
chisqsolhtml += 'A chi-squared test with 6 degrees of freedom was performed resulting in a test statistic of ' + pearval_str + '. '
if pearasig2_num < 0.001:
    chisqsolhtml += 'This results in an asymptotic p value which is ' + pearasig2_str + ' (reported as p &lt; .001). and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
else:
    if pearasig2_num > 0.05:
        chisqsolhtml += 'This results in an asymptotic p value which is ' + pearasig2_str + ' and therefore we do not have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
    else:
        chisqsolhtml += 'This results in an asymptotic p value which is ' + pearasig2_str + ' and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
chisqsolhtml += '</p>'
if exactopts:
    chisqsolhtml += '<ul><li>Question: Does the exact Chi-squared test agree with the asymptotic result?</li></ul>'
    chisqsolhtml += '<p>'
    chisqsolhtml += 'Solution: The output from SPSS is as follows:'
    chisqsolhtml += '</p>'
    chisqsolhtml += outputs['Table_Chi Square Tests.htm'].content
    chisqsolhtml += '<p>'
    chisqsolhtml += 'The chi-squared test is an approximate test but it is also possible to fit an exact test which involves looking at all possible tables with the same row and column totals and establishing how likely the observed table is. '
    chisqsolhtml += 'This is quoted as a p value and here the value is ' + pearesig2_str + '. '
    if pearasig2_num > 0.05 and pearesig2_num > 0.05:
        chisqsolhtml += 'Here the exact test agrees with the asymptotic result that we cannot reject the null hypothesis. '
    if pearasig2_num <= 0.05 and pearesig2_num <= 0.05:
        chisqsolhtml += 'Here the exact test agrees with the asymptotic result that we can reject the null hypothesis. '
    if pearasig2_num <= 0.05 and pearesig2_num > 0.05:
        chisqsolhtml += 'Here the exact test disagrees with the asymptotic result and states that we cannot in fact reject the null hypothesis. '
    if pearasig2_num > 0.05 and pearesig2_num <= 0.05:
        chisqsolhtml += 'Here the exact test disagrees with the asymptotic result and states that we can in fact reject the null hypothesis. '
    chisqsolhtml += '</p>'
if mh:
    chisqsolhtml += '<ul><li>Question: Does the Mantel-Haenszel test suggest a significant relationship between the (ordinal) variables?</li></ul>'
    chisqsolhtml += '<p>'
    chisqsolhtml += 'Solution: The output from SPSS is as follows:'
    chisqsolhtml += '</p>'
    chisqsolhtml += outputs['Table_Chi Square Tests.htm'].content
    chisqsolhtml += '<p>'
    chisqsolhtml += 'The Mantel-Haenszel test considers the two variables to be ordinal (ordered category) rather than nominal variables and again tests for a relationship with the null hypothesis being no relationship between the variables. '
    chisqsolhtml += 'SPSS gives the output for this test in the row marked Linear-by-Linear Association. As with the chi-squared test this test has a test statistic which here is ' + linassocval_str + ' and degrees of freedom ' + str(linassocdf) + '. '
    chisqsolhtml += 'This results in a p value given in SPSS of ' + linassocasig2_str + '. '
    if linassocasig2_num > 0.05:
        chisqsolhtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two variables are independent. '
    else:
        chisqsolhtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two variables are independent and there is therefore a linear relationship between the variables. '
    if linassocasig2_num < 0.001:
        chisqsolhtml += 'Although SPSS quotes the p value as ' + linassocval_str + ' it is not exactly zero and is in fact simply smaller than 0.001 as SPSS is quoting the first 3 decimal places. '
    if exactopts:
        chisqsolhtml += 'The Mantel-Haenszel test is an approximate test but it is also possible to fit an exact test which involves looking at all possible tables with the same row and column totals and establishing how likely the observed table is. '
        chisqsolhtml += 'This is quoted as a p value and here the value is ' + linassocesig2_str + '. '
        if linassocasig2_num > 0.05 and linassocesig2_num > 0.05:
            chisqsolhtml += 'Here the exact test agrees with the asymptotic result that we cannot reject the null hypothesis. '
        if linassocasig2_num <= 0.05 and linassocesig2_num <= 0.05:
            chisqsolhtml += 'Here the exact test agrees with the asymptotic result that we can reject the null hypothesis. '
        if linassocasig2_num <= 0.05 and linassocesig2_num > 0.05:
            chisqsolhtml += 'Here the exact test disagrees with the asymptotic result and states that we cannot in fact reject the null hypothesis. '
        if linassocasig2_num > 0.05 and linassocesig2_num <= 0.05:
            chisqsolhtml += 'Here the exact test disagrees with the asymptotic result and states that we can in fact reject the null hypothesis. '
    chisqsolhtml += '</p>'
outputs['chisquaredanswers.html'] = HTMLOutput(chisqsolhtml, "Chi-squared answers text")

conchtml = '<p>'
conchtml += 'In conclusion, we could report this to a reader as follows: '
conchtml += '</p>'
conchtml += '<p>'
conchtml += 'The relationship between the categorical variables, ' + varname1short + ' and ' + varname2short + ' was examined to look for associations. '
conchtml += 'A chi-squared test with ' + str(peardf) + ' degrees of freedom was performed resulting in a test statistic of ' + pearval_str + '. '
if exactopts:
    if pearesig2_num > 0.05:
        conchtml += 'This results in an exact p value of ' +  pearesig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent. '
    else:
        if pearesig2_num < 0.001:
            conchtml += 'This results in an exact p value which is less than 0.001 and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
        else:
            conchtml += 'This results in an exact p value of ' +  pearesig2_str + ' which is less than 0.05 and therefore we can reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
else:
    if pearasig2_num > 0.05:
        conchtml += 'This results in an asymptotic p value of ' +  pearasig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent. '
    else:
        if pearasig2_num < 0.001:
            conchtml += 'This results in an asymptotic p value which is less than 0.001 and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
        else:
            conchtml += 'This results in an asymptotic p value of ' +  pearasig2_str + ' which is less than 0.05 and therefore we can reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '

if mh:
    conchtml += '</p><p> As the two variables are ordinal, the Mantel-Haenszel test was in addition used to test for an association between ' + varname1short + ' and ' + varname2short + ' taking account of their ordinal nature. '
    conchtml += 'A Mantel-Haenszel test with ' + str(linassocdf) + ' degrees of freedom was performed resulting in a test statistic of ' + linassocval_str + '. '
    if exactopts:
        if linassocesig2_num > 0.05:
            conchtml += 'This results in an exact p value of ' +  linassocesig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent. '
        else:
            if linassocesig2_num < 0.001:
                conchtml += 'This results in an exact p value which is less than 0.001 and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
            else:
                conchtml += 'This results in an exact p value of ' +  linassocesig2_str + ' which is less than 0.05 and therefore we can reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
    else:
        if linassocasig2_num > 0.05:
            conchtml += 'This results in an asymptotic p value of ' +  linassocasig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent. '
        else:
            if linassocasig2_num < 0.001:
                conchtml += 'This results in an asymptotic p value which is less than 0.001 and therefore we have strong evidence to reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
            else:
                conchtml += 'This results in an asymptotic p value of ' +  linassocasig2_str + ' which is less than 0.05 and therefore we can reject the null hypothesis that ' + varname1short + ' and ' + varname2short + ' are independent and there is therefore some relationship between the variables. '
conchtml += '</p>'
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusions text")


combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['casesummary.html'].content
combinedprachtml += outputs['crosstabulation.html'].content
combinedprachtml += outputs['chisquared.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['casesummaryquestions.html'].content
combinedquesthtml += outputs['crosstabulationquestions.html'].content
combinedquesthtml += outputs['chisquaredquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['casesummaryanswers.html'].content
combinedsolhtml += outputs['crosstabulationanswers.html'].content
combinedsolhtml += outputs['chisquaredanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")
'''
    spssgui = r'''
\begin{aligned}
&\text{Select }\textbf{Descriptive Statistics}\text{ from the }\textbf{Analyze}\text{ menu.} \\ 
&\text{Select }\textbf{Crosstabs...}\text{ from the }\textbf{Descriptive Statistics}\text{ sub-menu.} \\ 
&\text{Click on the } \textbf{reset}\text{ button } \\ 
&\text{Copy the } \textbf{${datafile.variables[rowvar]['varlabel'] + ' [' + rowvar + ']'}}\text{ variables into the } \textbf{Row(s):}\text{ box. } \\ 
&\text{Copy the } \textbf{${datafile.variables[colvar]['varlabel'] + ' [' + colvar + ']'}}\text{ variable into the } \textbf{Column(s):}\text{ box. } \\ 
&\text{Click on the } \textbf{Statistics...}\text{ button. } \\ 
&\text{In the window that appears click on the } \textbf{Chi-square} \text{ tickbox. } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 

% if exactopts:
&\text{Click on the } \textbf{Exact...}\text{ button } \\
&\text{In the window that appears click on the } \textbf{${ Exact }}\text{ button. } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
% endif


% if cellopts:
<% done = False %> 
% if countopts:
% if not count:
&\text{Click on the } \textbf{Cells...} \text{ button.} \\ 
&\text{In the window that appears: } \\ 
<% done = True %> 
&\text{Under counts click on the } \textbf{Observed} \text{ tick box to remove it.} \\ 
% endif
% if expected:
% if not done:
&\text{Click on the } \textbf{Cells...} \text{ button.} \\ 
&\text{In the window that appears: } \\ 
<% done = True %> 
% endif
&\text{Under counts click on the } \textbf{Expected} \text{ tick box to include it.} \\ 
% endif
% endif
% if percentopts:
% if row:
% if not done:
&\text{Click on the } \textbf{Cells...} \text{ button.} \\ 
&\text{In the window that appears: } \\ 
<% done = True %> 
% endif
&\text{Under percentages click on the } \textbf{Row} \text{ tick box to include it.} \\ 
% endif
% if col:
% if not done:
&\text{Click on the } \textbf{Cells...} \text{ button.} \\ 
&\text{In the window that appears: } \\ 
<% done = True %> 
% endif
&\text{Under percentages click on the } \textbf{Column} \text{ tick box to include it.} \\ 
% endif
% if total:
% if not done:
&\text{Click on the } \textbf{Cells...} \text{ button.} \\ 
&\text{In the window that appears: } \\ 
<% done = True %> 
% endif
&\text{Under percentages click on the } \textbf{Total} \text{ tick box to include it.} \\ 
% endif
% endif
% if done: 
&\text{Click on the } \textbf{Continue}\text{ button } \\  
% endif
% endif
&\text{Click on the } \textbf{OK}\text{ button } \\  
\end{aligned}  
'''




